<?php

namespace Drupal\drd_pi;

use Drupal\Core\Config\Entity\ConfigEntityBase;
use mikehaertl\shellcommand\Command as ShellCommand;

/**
 * Provides an interface for defining Account entities.
 */
abstract class DrdPiAccount extends ConfigEntityBase implements DrdPiAccountInterface {

  /** @var string */
  protected $id;

  /** @var string */
  protected $label;

  /** @var \Drupal\drd\Logging */
  protected $logging;

  /** @var string */
  protected $lastShellOutput;

  /** @var DrdPiHost[] */
  protected $hosts;

  /** @var DrdPiCore[] */
  protected $cores;

  /** @var DrdPiDomain[] */
  protected $domains;

  /** @var \Drupal\Core\Config\ImmutableConfig */
  protected $config;

  /**
   * @inheritDoc
   */
  public function __construct(array $values, $entity_type) {
    parent::__construct($values, $entity_type);
    $this->logging = \Drupal::service('drd.logging');
    $this->config = \Drupal::config($this->getConfigName());
  }

  /**
   * @param DrdPiEntityInterface[] $platform
   * @param string $type
   * @param DrdPiEntityInterface $parent
   */
  protected function syncEntities(&$platform, $type, $parent = NULL) {

    // Get all internal entities
    $properties = [
      'pi_type' => $this->entityTypeId,
      'pi_account' => $this->id(),
    ];
    switch ($type) {
      case 'core':
        $properties['pi_id_host'] = $parent->id();
        break;

      case 'domain':
        $properties['pi_id_host'] = $parent->host()->id();
        $properties['pi_id_core'] = $parent->id();
        break;

    }
    $storage = \Drupal::entityTypeManager()->getStorage('drd_' . $type);
    /** @var \Drupal\drd\Entity\BaseInterface[] $internal */
    $internal = $storage->loadByProperties($properties);

    // Work through all platform entities
    foreach ($platform as &$entity) {

      $this->logging->debug('Checking @label', ['@label' => $entity->label()]);
      // Check if we already know that platform entity
      foreach ($internal as $drd_entity) {
        if (empty($drd_entity->drd_pi_available) && $drd_entity->get('pi_id_' . $type)->value == $entity->id()) {
          $this->logging->debug('- already available');
          $entity->setDrdEntity($drd_entity);
          $drd_entity->drd_pi_available = TRUE;
          break;
        }
      }

      // Create new DRD entity if don't have it yet
      if (!$entity->hasDrdEntity()) {
        $this->logging->debug('- create');
        $entity->create();
      }

    }

    // Enable/disable DRD entities that no longer exist on the platform
    foreach ($internal as $drd_entity) {
      $status = !empty($drd_entity->drd_pi_available);
      if ($drd_entity->isPublished() !== $status) {
        $this->logging->debug('@action @type @label', [
          '@action' => ($status ? 'Re-enable' : 'Disable'),
          '@type' => $type,
          '@label' => $drd_entity->label(),
        ]);
        $drd_entity
          ->setPublished($status)
          ->save();
      }
    }
  }

  /**
   * @inheritDoc
   */
  public function sync() {
    $this->logging->log('info', 'Receiving hosts');
    $hosts = $this->getPlatformHosts();
    $this->logging->log('info', 'Syncing hosts');
    $this->syncEntities($hosts, 'host');

    foreach ($hosts as $host) {
      $this->logging->log('info', 'Receiving cores for host @label', ['@label' => $host->label()]);
      $cores = $this->getPlatformCores($host);
      $this->logging->log('info', 'Syncing cores');
      $this->syncEntities($cores, 'core', $host);

      foreach ($cores as $core) {
        $this->logging->log('info', 'Receiving domains for core @label', ['@label' => $core->label()]);
        $domains = $this->getPlatformDomains($core);
        $this->logging->log('info', 'Syncing domains');
        $this->syncEntities($domains, 'domain', $core);
      }
    }

  }

  /**
   * @inheritDoc
   */
  public function getPlatformDomains($core) {
    $this->domains = [];

    if (isset($this->cores[$core->id()])) {
      $this->domains = $this->cores[$core->id()]->getDomains();
    }
    return $this->domains;
  }

  /**
   * @param string $cmd
   *
   * @return int
   */
  public function shell($cmd) {
    $this->lastShellOutput = '';
    $command = new ShellCommand($cmd);
    $command->execute();
    $this->lastShellOutput = $command->getOutput();
    return $command->getExitCode();
  }

}
