<?php

namespace Drupal\drd\Commands;

use Drupal\user\Entity\User;
use Drush\Commands\DrushCommands;
use Drush\Drush;

/**
 * Class Base.
 *
 * @package Drupal\drd
 */
class DrdCommands extends DrushCommands {

  /** @var \Drupal\drd\Plugin\Action\BaseInterface */
  protected $action;

  /** @var  string */
  protected $action_key;

  /** @var array */
  protected $options = [];

  /** @var \Drupal\Core\Entity\EntityInterface[] */
  protected $entities;

  public function __construct() {
    parent::__construct();
    if (drush_get_context('DRUSH_DEBUG')) {
      \Drupal::service('drd.logging')->enforceDebug();
    }
  }

  /**
   * @param \Consolidation\AnnotatedCommand\CommandData $commandData
   * @throws \Exception
   */
  protected function validateArguments($commandData, $arguments) {
    $commandArguments = $commandData->arguments();
    foreach ($arguments as $argument) {
      if (empty($commandArguments[$argument])) {
        throw new \Exception('Missing argument ' . $argument);
      }
    }
  }

  /**
   * @return \Drupal\drd\Entities
   */
  protected function service() {
    return \Drupal::service('drd.entities')
      ->setHostId($this->options['host-id'])
      ->setCoreId($this->options['core-id'])
      ->setDomainId($this->options['domain-id']);
  }

  /**
   * @param array $arguments
   * @param array $options
   *
   * @return $this
   */
  protected function prepare($arguments = [], $options = []) {
    \Drupal::service('drd.logging')->setIO($this->io());
    $this->action = \Drupal\drd\Plugin\Action\Base::instance($this->action_key);
    if (!$this->action || !($this->action instanceof \Drupal\drd\Plugin\Action\BaseInterface)) {
      return $this;
    }
    \Drupal::currentUser()->setAccount(User::load(1));

    foreach ($arguments as $key => $value) {
      $this->action->setActionArgument($key, $value);
    }

    foreach ($options as $key) {
      if (isset($this->options[$key])) {
        $this->action->setActionArgument($key, $this->options[$key]);
      }
    }

    return $this;
  }

  /**
   * Callback to execute the prepared action.
   */
  protected function execute() {
    if (empty($this->action)) {
      return;
    }
    if ($this->action instanceof \Drupal\drd\Plugin\Action\BaseGlobalInterface) {
      $this->entities[] = FALSE;
    }
    elseif (empty($this->entities)) {
      return;
    }

    $ok = $failure = 0;
    $this->io()->title('Executing ' . $this->action->getPluginDefinition()['label']);
    foreach ($this->entities as $entity) {
      if ($entity) {
        /** @var \Drupal\drd\Entity\BaseInterface $entity */
        $this->io()->section('- on id ' . $entity->id() . ': ' . $entity->getName());
        $result = $this->action->executeAction($entity);
      }
      else {
        $result = $this->action->executeAction();
      }
      if ($result !== FALSE) {
        $this->io()->success('  ok!');
        $ok++;
      }
      else {
        $this->io()->error('  failure!');
        $failure++;
      }

      $output = $this->action->getOutput();
      if ($output) {
        foreach ($output as $value) {
          $this->io()->write('  ' . $value, TRUE);
        }
      }
    }

    if (empty($failure)) {
      $this->io()->success('Completed!');
    }
    elseif (empty($ok)) {
      $this->io()->error('Completed!');
    }
    else {
      $this->io()->warning('Completed!');
    }
  }

  /**
   * Retrieve blocks from remote domain(s).
   *
   * @command drd:blocks
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @option module The remote module from which to receive blocks
   * @option delta The identifier of a specific block within a module
   * @aliases drd-blocks
   *
   * @param array $options
   */
  public function blocks($options = ['host-id' => null, 'core-id' => null, 'domain-id' => null, 'module' => null, 'delta' => null]) {
    $this->action_key = 'drd_action_blocks';
    $this->options = $options;
    $this->entities = $this->service()->domains();
    $this
      ->prepare([], ['module', 'delta'])
      ->execute();
  }

  /**
   * Run cron on remote domain(s).
   *
   * @command drd:cron
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @aliases drd-cron
   */
  public function cron($options = ['host-id' => null, 'core-id' => null, 'domain-id' => null]) {
    $this->action_key = 'drd_action_cron';
    $this->options = $options;
    $this->entities = $this->service()->domains();
    $this
      ->prepare()
      ->execute();
  }

  /**
   * Download database from remote domain(s).
   *
   * @command drd:database
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @aliases drd-database
   */
  public function database($options = ['host-id' => null, 'core-id' => null, 'domain-id' => null]) {
    $this->action_key = 'drd_action_database';
    $this->options = $options;
    $this->entities = $this->service()->domains();
    $this
      ->prepare()
      ->execute();
  }

  /**
   * Trx to determine all IP addresses of remote host(s).
   *
   * @command drd:dnslookup
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @aliases drd-dnslookup
   */
  public function dnslookup($options = ['host-id' => null, 'core-id' => null, 'domain-id' => null]) {
    $this->action_key = 'drd_action_dnslookup';
    $this->options = $options;
    $this->entities = $this->service()->hosts();
    $this
      ->prepare()
      ->execute();
  }

  /**
   * Change the settings of a local domain entity.
   *
   * @command drd:domainchange
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @option domain
   * @option secure
   * @option port
   * @option force
   * @aliases drd-domainchange
   */
  public function domainchange($options = ['host-id' => null, 'core-id' => null, 'domain-id' => null, 'domain' => null, 'secure' => null, 'port' => null, 'force' => FALSE]) {
    $this->action_key = 'drd_action_domainchange';
    $this->options = $options;
    $this->entities = $this->service()->domains();
    $this
      ->prepare([], ['domain', 'secure', 'port', 'force'])
      ->execute();
  }

  /**
   * Move a local domain record to a different core.
   *
   * @command drd:domainmove
   * @param int $dest_core_id
   *  Destination Core ID
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @aliases drd-domainmove
   */
  public function domainmove($dest_core_id, $options = ['host-id' => null, 'core-id' => null, 'domain-id' => null]) {
    $this->action_key = 'drd_action_domainchange';
    $this->options = $options;
    $this->entities = $this->service()->domains();
    $this
      ->prepare(['dest-core-id' => $dest_core_id], ['domain', 'secure', 'port', 'force'])
      ->execute();
  }

  /**
   * @hook validate drd:domainmove
   * @param \Consolidation\AnnotatedCommand\CommandData $commandData
   * @throws \Exception
   */
  public function validateDomainMove($commandData) {
    $this->validateArguments($commandData, ['dest_core_id']);
  }

  /**
   * Enable all domains of remote core(s).
   *
   * @command drd:domains:enableall
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @aliases drd-domains-enableall
   */
  public function domainsEnableall($options = ['host-id' => null, 'core-id' => null, 'domain-id' => null]) {
    $this->action_key = 'drd_action_domains_enableall';
    $this->options = $options;
    $this->entities = $this->service()->cores();
    $this
      ->prepare()
      ->execute();
  }

  /**
   * Receive all domains from remote core(s).
   *
   * @command drd:domains:receive
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @aliases drd-domains-receive
   */
  public function domainsReceive($options = ['host-id' => null, 'core-id' => null, 'domain-id' => null]) {
    $this->action_key = 'drd_action_domains_receive';
    $this->options = $options;
    $this->entities = $this->service()->cores();
    $this
      ->prepare()
      ->execute();
  }

  /**
   * Download a file from remote domain(s).
   *
   * @command drd:download
   * @param $source
   * @param $destination
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @aliases drd-download
   */
  public function download($source, $destination, $options = ['host-id' => null, 'core-id' => null, 'domain-id' => null]) {
    $this->action_key = 'drd_action_download';
    $this->options = $options;
    $this->entities = $this->service()->domains();
    $this
      ->prepare(['source' => $source, 'destination' => $destination])
      ->execute();
  }

  /**
   * @hook validate drd:download
   * @param \Consolidation\AnnotatedCommand\CommandData $commandData
   * @throws \Exception
   */
  public function validateDownload($commandData) {
    $this->validateArguments($commandData, ['source', 'destination']);
  }

  /**
   * Download error logs from remote domain(s).
   *
   * @command drd:errorlogs
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @aliases drd-errorlogs
   */
  public function errorlogs($options = ['host-id' => null, 'core-id' => null, 'domain-id' => null]) {
    $this->action_key = 'drd_action_error_logs';
    $this->options = $options;
    $this->entities = $this->service()->domains();
    $this
      ->prepare()
      ->execute();
  }

  /**
   * Flush all caches on remote domain(s).
   *
   * @command drd:flushcache
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @aliases drd-flushcache
   */
  public function flushcache($options = ['host-id' => null, 'core-id' => null, 'domain-id' => null]) {
    $this->action_key = 'drd_action_flush_cache';
    $this->options = $options;
    $this->entities = $this->service()->domains();
    $this
      ->prepare()
      ->execute();
  }

  /**
   * Receive information from remote domain(s).
   *
   * @command drd:info
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @aliases drd-info
   */
  public function info($options = ['host-id' => null, 'core-id' => null, 'domain-id' => null]) {
    $this->action_key = 'drd_action_info';
    $this->options = $options;
    $this->entities = $this->service()->domains();
    $this
      ->prepare()
      ->execute();
  }

  /**
   * Run the job scheduler on remote domain(s).
   *
   * @command drd:jobscheduler
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @aliases drd-jobscheduler
   */
  public function jobscheduler($options = ['host-id' => null, 'core-id' => null, 'domain-id' => null]) {
    $this->action_key = 'drd_action_job_scheduler';
    $this->options = $options;
    $this->entities = $this->service()->domains();
    $this
      ->prepare()
      ->execute();
  }

  /**
   * Domanload and update translations on remote domain(s).
   *
   * @command drd:l10n:update
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @aliases drd-l10n-update
   */
  public function l10nUpdate($options = ['host-id' => null, 'core-id' => null, 'domain-id' => null]) {
    $this->action_key = 'drd_action_update_translation';
    $this->options = $options;
    $this->entities = $this->service()->domains();
    $this
      ->prepare()
      ->execute();
  }

  /**
   * Push the latest library to remote domain(s).
   *
   * @command drd:library:push
   * @param string $source
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @aliases drd-library-push
   */
  public function libraryPush($source, $options = ['host-id' => null, 'core-id' => null, 'domain-id' => null]) {
    $this->action_key = 'drd_action_library';
    $this->options = $options;
    $this->entities = $this->service()->domains();
    $this
      ->prepare(['source' => $source])
      ->execute();
  }

  /**
   * @hook validate drd:library:push
   * @param \Consolidation\AnnotatedCommand\CommandData $commandData
   * @throws \Exception
   */
  public function validateLibraryPush($commandData) {
    $this->validateArguments($commandData, ['source']);
  }

  /**
   * Re-build the local library.
   *
   * @command drd:library:build
   * @aliases drd-library-build
   */
  public function libraryBuild() {
    $this->action_key = 'drd_action_library_build';
    $this
      ->prepare()
      ->execute();
  }

  /**
   * Get or set the maintenance mode on/from remote domain(s).
   *
   * @command drd:maintenancemode
   * @param string $mode
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @aliases drd-maintenancemode
   */
  public function maintenancemode($mode, $options = ['host-id' => null, 'core-id' => null, 'domain-id' => null]) {
    $this->action_key = 'drd_action_maintenance_mode';
    $this->options = $options;
    $this->entities = $this->service()->domains();
    $this
      ->prepare(['mode' => $mode])
      ->execute();
  }

  /**
   * @hook validate drd:maintenancemode
   * @param \Consolidation\AnnotatedCommand\CommandData $commandData
   * @throws \Exception
   */
  public function validateMaintenanceMode($commandData) {
    $this->validateArguments($commandData, ['mode']);
  }

  /**
   * Execute arbitrary PHP code on remote domain(s).
   *
   * @command drd:php
   * @param string $php
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @aliases drd-php
   */
  public function php($php, $options = ['host-id' => null, 'core-id' => null, 'domain-id' => null]) {
    $this->action_key = 'drd_action_php';
    $this->options = $options;
    $this->entities = $this->service()->domains();
    $this
      ->prepare(['php' => $php])
      ->execute();
  }

  /**
   * @hook validate drd:php
   * @param \Consolidation\AnnotatedCommand\CommandData $commandData
   * @throws \Exception
   */
  public function validatePhp($commandData) {
    $this->validateArguments($commandData, ['php']);
  }

  /**
   * Ping remote domain(s).
   *
   * @command drd:ping
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @option save
   * @aliases drd-ping
   */
  public function ping($options = ['host-id' => null, 'core-id' => null, 'domain-id' => null, 'save' => FALSE]) {
    $this->action_key = 'drd_action_ping';
    $this->options = $options;
    $this->entities = $this->service()->domains();
    $this
      ->prepare([], ['save'])
      ->execute();
  }

  /**
   * Receive a list of projects used by remote domain(s).
   *
   * @command drd:projects:usage
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @aliases drd-projects-usage
   */
  public function projectsUsage($options = ['host-id' => null, 'core-id' => null, 'domain-id' => null]) {
    $this->action_key = 'drd_action_projects';
    $this->options = $options;
    $this->entities = $this->service()->domains();
    $this
      ->prepare()
      ->execute();
  }

  /**
   * Check update status for all projects being used by all remote domains.
   *
   * @command drd:projects:status
   * @aliases drd-projects-status
   */
  public function projectsStatus() {
    $this->action_key = 'drd_action_projects_status';
    $this
      ->prepare()
      ->execute();
  }

  /**
   * Update a complete Drupal installation of remote core(s).
   *
   * @command drd:projects:update
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @option dry-run Perform the update in the working directory but do not commit, push or deploy
   * @option show-log Show the log output
   * @aliases drd-projects-update
   */
  public function projectsUpdate($options = ['host-id' => null, 'core-id' => null, 'domain-id' => null, 'dry-run' => FALSE, 'show-log' => FALSE]) {
    $this->action_key = 'drd_action_projects_update';
    $this->options = $options;
    $this->entities = $this->service()->cores();
    $this
      ->prepare([], ['dry-run', 'show-log'])
      ->execute();
  }

  /**
   * Display logs of core updates.
   *
   * @command drd:projects:update:log
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @option id Shows the latest by default, you can get any of the other from the list with this option
   * @option list List die available logs
   * @aliases drd-projects-update-log
   */
  public function projectsUpdateLog($options = ['host-id' => null, 'core-id' => null, 'domain-id' => null, 'id' => null, 'list' => FALSE]) {
    $this->action_key = 'drd_action_projects_update_log';
    $this->options = $options;
    $this->entities = $this->service()->cores();
    $this
      ->prepare([], ['id', 'list'])
      ->execute();
  }

  /**
   * Receive a URL to start a session on remote domain(s).
   *
   * @command drd:session
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @aliases drd-session
   */
  public function session($options = ['host-id' => null, 'core-id' => null, 'domain-id' => null]) {
    $this->action_key = 'drd_action_session';
    $this->options = $options;
    $this->entities = $this->service()->domains();
    $this
      ->prepare()
      ->execute();
  }

  /**
   * Run update.php on remote domain(s).
   *
   * @command drd:update
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @aliases drd-update
   */
  public function update($options = ['host-id' => null, 'core-id' => null, 'domain-id' => null]) {
    $this->action_key = 'drd_action_update';
    $this->options = $options;
    $this->entities = $this->service()->domains();
    $this
      ->prepare()
      ->execute();
  }

  /**
   * Change credentials of a user on remote domain(s).
   *
   * @command drd:user:credentials
   * @param int $uid
   * @option host-id Host ID
   * @option core-id Core ID
   * @option domain-id Domain ID
   * @option username The user name to be set
   * @option password The password to be set
   * @option status The status to be set
   * @aliases drd-user-credentials
   */
  public function userCredentials($uid, $options = ['host-id' => null, 'core-id' => null, 'domain-id' => null, 'username' => null, 'password' => null, 'status' => null]) {
    $this->action_key = 'drd_action_user_credentials';
    $this->options = $options;
    $this->entities = $this->service()->domains();
    $this
      ->prepare(['uid' => $uid, ['username', 'password', 'status']])
      ->execute();
  }

  /**
   * @hook validate drd:credentials
   * @param \Consolidation\AnnotatedCommand\CommandData $commandData
   * @throws \Exception
   */
  public function validateUserCredentials($commandData) {
    $this->validateArguments($commandData, ['uid']);
  }

  /**
   * Remove unused releases from the local database.
   *
   * @command drd:cleanup:unused:releases
   * @aliases drd-cleanup-unused-releases
   */
  public function cleanupUnusedReleases() {
    \Drupal::service('drd.cleanup')->cleanupReleases();
  }

  /**
   * Remove unused major versions from the local database.
   *
   * @command drd:cleanup:unused:majors
   * @aliases drd-cleanup-unused-majors
   */
  public function cleanupUnusedMajors() {
    \Drupal::service('drd.cleanup')->cleanupMajors();
  }

  /**
   * Remove unused projects from the local database.
   *
   * @command drd:cleanup:unused:projects
   * @aliases drd-cleanup-unused-projects
   */
  public function cleanupUnusedProjects() {
    \Drupal::service('drd.cleanup')->cleanupProjects();
  }

  /**
   * Remove all projects, major versions and releases from the local database.
   *
   * @command drd:reset:all:projects:data
   * @aliases drd-reset-all-projects-data
   */
  public function resetAllProjectsData() {
    \Drupal::service('drd.cleanup')->resetAll();
  }

}
