<?php

namespace Drupal\drd\Plugin\Update\Storage;

use Drupal\Core\Form\FormStateInterface;
use Drupal\drd\Plugin\Update\Base as UpdateBase;
use Drupal\drd\Update\PluginStorageInterface;
use Symfony\Component\Filesystem\Filesystem;

/**
 * Abstract DRD Update plugin to implement general functionality.
 */
abstract class Base extends UpdateBase implements PluginStorageInterface {

  /** @var FileSystem */
  protected $fs;

  /** @var string */
  protected $drupalDirectory;

  /** @var string */
  protected $workingDirectory;

  /** @var string */
  private $logText = '';

  /** @var \Drupal\drd\Update\PluginBuildInterface */
  private $buildPlugin;

  /** @var \Drupal\drd\Update\PluginProcessInterface */
  private $processPlugin;

  /** @var \Drupal\drd\Update\PluginTestInterface */
  private $testPlugin;

  /** @var \Drupal\drd\Update\PluginDeployInterface */
  private $deployPlugin;

  /** @var \Drupal\drd\Update\PluginFinishInterface */
  private $finishPlugin;

  /** @var \Drupal\drd\Entity\CoreInterface */
  private $core;

  /**
   * @inheritDoc
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->fs = new Filesystem();
  }

  /**
   * @inheritDoc
   */
  public function defaultConfiguration() {
    return [
        'drupalroot' => '',
      ] + parent::defaultConfiguration();
  }

  /**
   * @inheritDoc
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $element = parent::buildConfigurationForm($form, $form_state);

    $element['drupalroot'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Drupal Root'),
      '#default_value' => $this->configuration['drupalroot'],
      '#description' => $this->t('Relative path to Drupal root directory from the working directory without leading or trailing slash.'),
      '#weight' => 80,
    ];

    return $element;
  }

  /**
   * @inheritDoc
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->configuration['drupalroot'] = trim($this->getFormValue($form_state, 'drupalroot'), '/');
  }

  /**
   * @inheritDoc
   */
  final public function stepPlugins($build, $process, $test, $deploy, $finish) {
    $this->buildPlugin = $build;
    $this->processPlugin = $process;
    $this->testPlugin = $test;
    $this->deployPlugin = $deploy;
    $this->finishPlugin = $finish;
  }

  /**
   * @inheritDoc
   */
  public function scriptHooks() {
    return [
      'preUpdate' => $this->t('At the very beginning'),
      'postUpdate' => $this->t('At the very end'),
      'prePrepare' => $this->t('Before preparing working directory'),
      'postPrepare' => $this->t('After preparing working directory'),
      'preSave' => $this->t('Before saving working firectory'),
      'postSave' => $this->t('After saving working firectory'),
    ] + parent::scriptHooks();
  }

  /**
   * @inheritDoc
   */
  final public function execute($core, $releases, $dry, $showlog) {
    $this->core = $core;
    $result = TRUE;
    try {
      $this
        ->log('Start')
        ->executeScript($this, 'preUpdate')
        ->executeScript($this, 'prePlugin')
        ->setWorkingDirectory()
        ->executeScript($this, 'prePrepare')
        ->prepareWorkingDirectory()
        ->executeScript($this, 'postPrepare');
      $this->buildPlugin
        ->executeScript($this, 'prePlugin')
        ->build($this, $releases)
        ->patch($this)
        ->executeScript($this, 'postPlugin');
      if ($this->buildPlugin->hasChanged()) {
        $this->processPlugin
          ->executeScript($this, 'prePlugin')
          ->process($this)
          ->executeScript($this, 'postPlugin');
      }
      if ($this->processPlugin->hasSucceeded()) {
        $this->testPlugin
          ->executeScript($this, 'prePlugin')
          ->test($this)
          ->executeScript($this, 'postPlugin');
      }
      if ($dry) {
        if ($this->testPlugin->hasSucceeded()) {
          $this->deployPlugin
            ->executeScript($this, 'prePlugin')
            ->dryRun($this)
            ->executeScript($this, 'postPlugin');
        }
        if ($this->deployPlugin->hasSucceeded()) {
          $this->finishPlugin
            ->executeScript($this, 'prePlugin')
            ->dryRun($this)
            ->executeScript($this, 'postPlugin');
        }
      }
      else {
        if ($this->testPlugin->hasSucceeded()) {
          $this->deployPlugin
            ->executeScript($this, 'prePlugin')
            ->deploy($this)
            ->executeScript($this, 'postPlugin');
        }
        if ($this->deployPlugin->hasSucceeded()) {
          $this->finishPlugin
            ->executeScript($this, 'prePlugin')
            ->finish($this)
            ->executeScript($this, 'postPlugin');
        }
      }
    }
    catch (\Exception $ex) {
      $result = 'Exception: ' . $ex->getMessage();
      $this->log($result);
    }

    if ($dry) {
      $this->log('Finished dry');
    }
    else {
      try {
        $this->finishPlugin->cleanup($this);
        $this->deployPlugin->cleanup($this);
        $this->testPlugin->cleanup($this);
        $this->processPlugin->cleanup($this);
        $this->buildPlugin->cleanup($this);
        if ($result === TRUE) {
          $this
            ->executeScript($this, 'preSave')
            ->saveWorkingDirectory()
            ->executeScript($this, 'postSave');
        }
        $this
          ->cleanup($this)
          ->executeScript($this, 'postPlugin')
          ->executeScript($this, 'postUpdate')
          ->log('Finish');
      }
      catch (\Exception $ex) {
        $result = 'Exception during save and cleanup: ' . $ex->getMessage();
        $this->log($result);
      }
    }

    if ($showlog) {
      print($this->logText);
    }
    $this->core->saveUpdateLog($this->logText);
    return $result;
  }

  /**
   * @inheritDoc
   */
  final public function log($log) {
    $logs = is_string($log) ? [$log] : $log;
    foreach ($logs as $log) {
      if (!empty(trim($log))) {
        $t = \Drupal::service('date.formatter')
          ->format(time(), 'custom', 'Y-m-d H:i:s');
        $this->logText .= '[' . $t . '] ' . str_replace("\n", "    \n", $log) . "\n";
      }
    }
    return $this;
  }

  /**
   * @inheritDoc
   */
  final public function getCore() {
    return $this->core;
  }

  /**
   * @inheritDoc
   */
  public function getDrupalDirectory() {
    return empty($this->configuration['drupalroot']) ?
      $this->workingDirectory :
      $this->workingDirectory . DIRECTORY_SEPARATOR . $this->configuration['drupalroot'];
  }

  /**
   * @inheritDoc
   */
  final public function getWorkingDirectory() {
    return $this->workingDirectory;
  }

  /**
   * @inheritDoc
   */
  final public function getBuildPlugin() {
    return $this->buildPlugin;
  }

  /**
   * @inheritDoc
   */
  final public function getProcessPlugin() {
    return $this->processPlugin;
  }

  /**
   * @inheritDoc
   */
  final public function getTestPlugin() {
    return $this->testPlugin;
  }

  /**
   * @inheritDoc
   */
  final public function getDeployPlugin() {
    return $this->deployPlugin;
  }

  /**
   * @inheritDoc
   */
  final public function getFinishPlugin() {
    return $this->finishPlugin;
  }

  /**
   * @inheritDoc
   */
  public function setWorkingDirectory() {
    if (!isset($this->workingDirectory)) {
      $this->workingDirectory = $this->fs->tempnam(file_directory_temp(), 'drd-update-');
      $this->drupalDirectory = $this->workingDirectory . DIRECTORY_SEPARATOR . $this->configuration['drupalroot'];
    }
    return $this;
  }

  /**
   * @inheritDoc
   */
  public function prepareWorkingDirectory() {
    $this->fs->remove($this->workingDirectory);
    $this->fs->mkdir($this->workingDirectory);
    return $this;
  }

  /**
   * @inheritDoc
   */
  public function saveWorkingDirectory() {
    return $this;
  }

  /**
   * @inheritDoc
   */
  public function cleanup($storage) {
    $this->fs->remove($this->workingDirectory);
    return $this;
  }

}
